@@ -45,37 +45,58 @@ pub fn find_2d<A: Axis, T: Aabb, F: CollisionHandler<T>>(
     }
 }
 
-//Calls colliding on all aabbs that intersect between two groups and only one aabbs
-//that intsect
-//TODO turn into struct where the arguments are stored first except for the function.
-pub fn find_parallel_2d<A: Axis, T: Aabb>(
-    prevec1: &mut PreVec,
+pub struct FindParallel2DBuilder<'a, A: Axis, T: Aabb> {
+    prevec: &'a mut PreVec,
     axis: A,
-    bots1: AabbPin<&mut [T]>,
-    bots2: AabbPin<&mut [T]>,
-    mut func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>),
-) {
-    //let mut b: OtherAxisCollider<A, _> = OtherAxisCollider { a: &mut func, axis };
+    bots1: AabbPin<&'a mut [T]>,
+    bots2: AabbPin<&'a mut [T]>,
+}
+impl<'a, A: Axis, T: Aabb> FindParallel2DBuilder<'a, A, T> {
+    pub fn new(
+        prevec: &'a mut PreVec,
+        axis: A,
+        bots1: AabbPin<&'a mut [T]>,
+        bots2: AabbPin<&'a mut [T]>,
+    ) -> Self {
+        FindParallel2DBuilder {
+            prevec,
+            axis,
+            bots1,
+            bots2,
+        }
+    }
 
-    self::find_other_parallel3(prevec1, axis, (bots1, bots2), &mut func)
+    pub fn build(self, mut func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>)) {
+        self::find_other_parallel3(self.prevec, self.axis, (self.bots1, self.bots2), &mut func)
+    }
 }
 
-pub fn find_perp_2d1_once<A: Axis, T: Aabb>(
-    _prevec: &mut PreVec,
-    axis: A, //the axis of r2.
-    mut y: AabbPin<&mut T>,
-    mut r2: AabbPin<&mut [T]>,
-    mut func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>),
-) {
-    for y2 in r2.borrow_mut() {
-        //Exploit the sorted property, to exit early
-        if y.get().get_range(axis).end < y2.get().get_range(axis).start {
-            break;
-        }
+pub struct FindPerp2DBuilder<'a, A: Axis, T: Aabb> {
+    axis: A,
+    y: AabbPin<&'a mut T>,
+    r2: AabbPin<&'a mut [T]>,
+}
 
-        //Because we didnt exit from the previous comparison, we only need to check one thing.
-        if y.get().get_range(axis).start <= y2.get().get_range(axis).end {
-            func(y.borrow_mut(), y2);
+impl<'a, A: Axis, T: Aabb> FindPerp2DBuilder<'a, A, T> {
+    pub fn new(axis: A, y: AabbPin<&'a mut T>, r2: AabbPin<&'a mut [T]>) -> Self {
+        FindPerp2DBuilder { axis, y, r2 }
+    }
+    pub fn build(self, mut func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>)) {
+        let FindPerp2DBuilder {
+            axis,
+            mut y,
+            mut r2,
+        } = self;
+        for y2 in r2.borrow_mut() {
+            //Exploit the sorted property, to exit early
+            if y.get().get_range(axis).end < y2.get().get_range(axis).start {
+                break;
+            }
+
+            //Because we didnt exit from the previous comparison, we only need to check one thing.
+            if y.get().get_range(axis).start <= y2.get().get_range(axis).end {
+                func(y.borrow_mut(), y2);
+            }
         }
     }
 }
